local overlays = {}
local auras = {}

local function formatTime( seconds )
	local tempTime;
	if ( seconds >= 86400  ) then
		tempTime = floor(seconds / 86400);
		return "%dd", tempTime;
	end
	if ( seconds >= 3600  ) then
		tempTime = floor(seconds / 3600);
		return "%dh", tempTime;
	end
	if ( seconds >= 60  ) then
		tempTime = floor(seconds / 60);
		return "%dm", tempTime;
	end
	if ( seconds >= 10  ) then
		tempTime = floor(seconds);
		return "%ds", tempTime;
	end
	tempTime = floor(seconds*10)/10;
	return "%.1f", tempTime;
end

local f = CreateFrame("Frame")

local function GetSlotCooldown(slot)
	local now = GetTime()
	local cooldownStart, cooldownLength = GetActionCooldown( slot )
	local endTime = cooldownStart + cooldownLength
	local realCooldown, energyCooldown, auraDuration = 0, 0, 0
	if endTime > 0 and cooldownLength > 1.5 then
		realCooldown = endTime - now
	end

	local btype, spellId = GetActionInfo(slot)
	if( btype == 'macro' ) then
		local name, rank = GetMacroSpell( spellId )
		if( name ) then
			if rank then
				name = name.."("..rank..")"
			end
			btype = 'spell';
			spellId = name;
		end
	end
	
	if( btype == 'spell' ) then
		local spellName, _, _, spellCost, _, powerType, castingTime = GetSpellInfo(spellId)
		if auras[spellName] then
			auraDuration = auras[spellName] - now
		end
		if UnitPowerType("player") == powerType then
			local currentPower = UnitPower("player", powerType)
			if spellCost > currentPower then
				local passiveRegen, activeRegen = GetPowerRegen()
				local regenRate = 0
				if UnitAffectingCombat("player") then
					regenRate = activeRegen
				else
					regenRate = passiveRegen
				end
				local powerDifference = spellCost - currentPower
				if regenRate >= 1 then
					energyCooldown = powerDifference/regenRate
				end
			end
		end
	end
	
	if realCooldown > energyCooldown and realCooldown > 0 then
		return realCooldown, "ff0000"
	elseif energyCooldown > 0 then
		return energyCooldown, "ffff00"
	else
		return auraDuration, "00ff00"
	end
end

function updateAuras()
	for k,v in pairs(auras) do
		auras[k] = nil
	end
	if(UnitExists("target")) then
		unit = "target"
	else
		unit = "player"
	end
	if UnitIsFriend("player", unit) then
		for i = 1,40 do
			local name, _, _, _, _, _, expires = UnitBuff(unit, i, "player")
			if(name) then
				if(expires > 0) then
					auras[name] = expires
				end
			else
				break
			end
		end
	else
		for i = 1,40 do
			local name, _, _, _, _, _, expires = UnitDebuff(unit, i, "player")
			if(name) then
				if(expires > 0) then
					auras[name] = expires
				end
			else
				break
			end
		end
		for i = 1,40 do
			local name, _, _, _, _, _, expires = UnitBuff("player", i, "player")
			if(name) then
				if(expires > 0) then
					auras[name] = expires
				end
			else
				break
			end
		end
	end
end

local function updateButtons()
	for _, overlay in ipairs(overlays) do
		--local slot = ActionButton_GetPagedID(overlay.button)
		local slot = overlay.button:GetAttribute("action") or ActionButton_GetPagedID(overlay.button)
		local cooldown, color = GetSlotCooldown(slot)
		if cooldown > 0 then
			local x,t = formatTime(cooldown)
			overlay.text:SetFormattedText("|cff"..color..x.."|r", t)
			if IsUsableAction(slot) and color == "00ff00" then
				overlay.frame.texture:SetVertexColor(0, 1, 0, 1)
			else
				overlay.frame.texture:SetVertexColor(1, 0, 0, 1)
			end
			if not overlay.frame:IsVisible() then
				overlay.frame:Show()
			end
		else
			if overlay.frame:IsVisible() then
				overlay.frame:Hide()
			end
			overlay.text:SetText()
		end
	end
end

function f:ACTIONBAR_SLOT_CHANGED()
	updateButtons()
end

function f:ACTIONBAR_UPDATE_COOLDOWN()
	updateButtons()
end

function f:UNIT_AURA()
	updateAuras()
end

function f:PLAYER_TARGET_CHANGED()
	updateAuras()
end

local timeSinceLastUpdate = 0
f:SetScript("OnUpdate", function(self, elapsed)
	timeSinceLastUpdate = timeSinceLastUpdate + elapsed
	if( timeSinceLastUpdate > 0.1 ) then
		updateButtons()
		timeSinceLastUpdate = 0
	end
end)

f:SetScript("OnEvent", function(self, event, ...) if self[event] then return self[event](self, ...) end end)



local ActionButtons = {}
local function GetActionButtons(...)
	for i = 1, select('#',...) do
		local frame = select(i, ...)
		if frame:GetNumChildren() > 0 then
			GetActionButtons(frame:GetChildren())
		end
		if frame:GetAttribute("type") == "action" then
			tinsert(ActionButtons, frame)
		end
	end
end

local function makeOverlays()
	for i, button in ipairs(ActionButtons) do
		if not button then return end
		if not button.CrackCooldownOverlay then
			local frame = CreateFrame("frame", nil, button)
			--frame:Hide()
			frame:SetAllPoints(button)
			frame:SetFrameLevel(button:GetFrameLevel() + 3)
			
			local text = frame:CreateFontString(nil, "OVERLAY")
			text:SetPoint("CENTER")
			--text:SetFont("Fonts\\FRIZQT__.TTF", 12, "OUTLINE")
			text:SetFont("Fonts\\ARIALN.TTF", 14, "OUTLINE")
			
			local action = button:GetAttribute("action") or ActionButton_GetPagedID(button) or "?"
			text:SetText(action)
			text:SetShadowOffset(1,-1)
			
			tinsert(overlays, {
				button = button,
				frame = frame,
				text = text,
			})
			
			-- hilight texture to test visibility of frame --
			local t = frame:CreateTexture(nil,"BACKGROUND");
			t:SetTexture("Interface\\Buttons\\CheckButtonHilight");
			t:SetAllPoints(frame);
			t:SetBlendMode("add");
			t:SetVertexColor(0, 0.25, 1, 1);
			frame.texture = t;
			-- end hilight texture --
			button.CrackCooldownOverlay = frame
		end
	end
end

local function ActionHook(self, attribute, slot)
	--print(self:GetName(), attribute, slot)
	if attribute ~= "action" or self.CrackCooldownOverlay then return end
	if HasAction(slot) then
		print(self:GetName(), GetActionInfo(slot))
		tinsert(ActionButtons, self)
		makeOverlays()
	end
end
hooksecurefunc(getmetatable(ActionButton1).__index, 'SetAttribute', ActionHook)

f:RegisterEvent("PLAYER_LOGIN")
function f:PLAYER_LOGIN()
	GetActionButtons(UIParent:GetChildren())
	f.TimeSinceLastUpdate = 0
	f:RegisterEvent("ACTIONBAR_SLOT_CHANGED")
	f:RegisterEvent("ACTIONBAR_PAGE_CHANGED")
	f:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN")
	f:RegisterEvent("UNIT_AURA")
	f:RegisterEvent("PLAYER_TARGET_CHANGED")
	makeOverlays()
end